/*++

Module Name:

    test.c

Abstract:

    Test program for CSE451 Project #1 "NT API Monitoring"

    This test program calls NtQuerySystemInformation specifing the CSE451
    information class and prints out the NT API results.

--*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <nt.h>
#include <ntrtl.h>
#include <nturtl.h>


VOID
PrintStatus (
    NTSTATUS Status
    )
{
    if (NT_INFORMATION(Status))  { printf("Information\n"); }
    else if (NT_WARNING(Status)) { printf("Warning\n"); }
    else if (NT_ERROR(Status))   { printf("Error\n"); }
    else                         { printf("Success\n"); }
}

VOID
__cdecl
main (argc, argv)
    int argc;
    char *argv[];
{
    NTSTATUS Status;
    PSYSTEM_CSE451_INFORMATION CSE451ApiData;
    ULONG BufferSize;
    ULONG Size;
    ULONG i;

    LARGE_INTEGER SystemTime, LocalTime;
    TIME_FIELDS TimeFields;

    BufferSize = 4096;
    if (argc > 1) {
        sscanf(argv[1], "%d", &BufferSize);
    }

    printf("Size of SYSTEM_CSE451_INFORMATION = %d\n", sizeof(SYSTEM_CSE451_INFORMATION));
    printf("FIELD_OFFSET Events[0] = %d\n", FIELD_OFFSET(SYSTEM_CSE451_INFORMATION,Events));

    (VOID) NtQuerySystemTime( &SystemTime );

    RtlTimeToTimeFields( &SystemTime, &TimeFields );
    printf("SystemTime = %4d/%02d/%02d  %2d:%02d:%02d.%03d\n", TimeFields.Year, TimeFields.Month, TimeFields.Day, TimeFields.Hour, TimeFields.Minute, TimeFields.Second, TimeFields.Milliseconds );

    (VOID) RtlSystemTimeToLocalTime( &SystemTime, &LocalTime );

    RtlTimeToTimeFields( &LocalTime, &TimeFields );
    printf("LocalTime  = %4d/%02d/%02d  %2d:%02d:%02d.%03d\n", TimeFields.Year, TimeFields.Month, TimeFields.Day, TimeFields.Hour, TimeFields.Minute, TimeFields.Second, TimeFields.Milliseconds );

    CSE451ApiData = malloc(BufferSize);

    printf("BufferSize = %d\n", BufferSize);

    if (!NT_SUCCESS(Status = NtQuerySystemInformation( SystemCSE451Information,
                                                       CSE451ApiData,
                                                       BufferSize,
                                                       &Size))) {

        printf("Error in calling NtQuerySystemInformation asking for SystemCSE451Information = %08lx\n", Status);
        return;
    }

    printf("Returned size = %d\n", BufferSize, Size);

    printf("API       Calls  Success   Inform     Warn    Error    Delta\n");
    printf("---       -----  -------   ------     ----    -----    -----\n");
    printf("Create %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Create[0], CSE451ApiData->Create[1], CSE451ApiData->Create[2], CSE451ApiData->Create[3], CSE451ApiData->Create[4], CSE451ApiData->Create[0]-CSE451ApiData->Create[1]-CSE451ApiData->Create[2]-CSE451ApiData->Create[3]-CSE451ApiData->Create[4]);
    printf("Close  %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Close[0],  CSE451ApiData->Close[1],  CSE451ApiData->Close[2],  CSE451ApiData->Close[3],  CSE451ApiData->Close[4],  CSE451ApiData->Close[0]-CSE451ApiData->Close[1]-CSE451ApiData->Close[2]-CSE451ApiData->Close[3]-CSE451ApiData->Close[4]);
    printf("Open   %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Open[0],   CSE451ApiData->Open[1],   CSE451ApiData->Open[2],   CSE451ApiData->Open[3],   CSE451ApiData->Open[4],   CSE451ApiData->Open[0]-CSE451ApiData->Open[1]-CSE451ApiData->Open[2]-CSE451ApiData->Open[3]-CSE451ApiData->Open[4]);
    printf("Read   %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Read[0],   CSE451ApiData->Read[1],   CSE451ApiData->Read[2],   CSE451ApiData->Read[3],   CSE451ApiData->Read[4],   CSE451ApiData->Read[0]-CSE451ApiData->Read[1]-CSE451ApiData->Read[2]-CSE451ApiData->Read[3]-CSE451ApiData->Read[4]);
    printf("Write  %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Write[0],  CSE451ApiData->Write[1],  CSE451ApiData->Write[2],  CSE451ApiData->Write[3],  CSE451ApiData->Write[4],  CSE451ApiData->Write[0]-CSE451ApiData->Write[1]-CSE451ApiData->Write[2]-CSE451ApiData->Write[3]-CSE451ApiData->Write[4]);
    printf("Get    %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Get[0],    CSE451ApiData->Get[1],    CSE451ApiData->Get[2],    CSE451ApiData->Get[3],    CSE451ApiData->Get[4],    CSE451ApiData->Get[0]-CSE451ApiData->Get[1]-CSE451ApiData->Get[2]-CSE451ApiData->Get[3]-CSE451ApiData->Get[4]);
    printf("Set    %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Set[0],    CSE451ApiData->Set[1],    CSE451ApiData->Set[2],    CSE451ApiData->Set[3],    CSE451ApiData->Set[4],    CSE451ApiData->Set[0]-CSE451ApiData->Set[1]-CSE451ApiData->Set[2]-CSE451ApiData->Set[3]-CSE451ApiData->Set[4]);
    printf("Query  %8d %8d %8d %8d %8d %8d\n", CSE451ApiData->Query[0],  CSE451ApiData->Query[1],  CSE451ApiData->Query[2],  CSE451ApiData->Query[3],  CSE451ApiData->Query[4],  CSE451ApiData->Query[0]-CSE451ApiData->Query[1]-CSE451ApiData->Query[2]-CSE451ApiData->Query[3]-CSE451ApiData->Query[4]);

    printf("NumberOfEvents = %d\n", CSE451ApiData->NumberOfEvents);
    for (i = 0; i < CSE451ApiData->NumberOfEvents; i++) {

        (VOID) RtlSystemTimeToLocalTime( &CSE451ApiData->Events[i].Time, &LocalTime );
        RtlTimeToTimeFields( &LocalTime, &TimeFields );

        printf("%4d/%02d/%02d  %2d:%02d:%02d.%03d = ", TimeFields.Year, TimeFields.Month, TimeFields.Day, TimeFields.Hour, TimeFields.Minute, TimeFields.Second, TimeFields.Milliseconds );

        switch (CSE451ApiData->Events[i].EventType) {
        case CreateCall:
            printf("Create Call\n");
            break;
        case CreateReturn:
            printf("Create Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case CloseCall:
            printf("Close Call\n");
            break;
        case CloseReturn:
            printf("Close Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case OpenCall:
            printf("Open Call\n");
            break;
        case OpenReturn:
            printf("Open Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case ReadCall:
            printf("Read Call\n");
            break;
        case ReadReturn:
            printf("Read Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case WriteCall:
            printf("Write Call\n");
            break;
        case WriteReturn:
            printf("Write Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case GetCall:
            printf("Get Call\n");
            break;
        case GetReturn:
            printf("Get Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case SetCall:
            printf("Set Call\n");
            break;
        case SetReturn:
            printf("Set Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case QueryCall:
            printf("Query Call\n");
            break;
        case QueryReturn:
            printf("Query Return "); PrintStatus(CSE451ApiData->Events[i].Status);
            break;
        case BufferOverflow:
            printf("Buffer Overflow\n");
            break;

        }
    }
}

